/* autocal.c
 *
 * By John Pavan
 *
 * For doing autocalibration of a histogram using a known source.
 */

/* --- includes --- */

#include <gtk/gth.h>
#include "gtkgraph.h"
#include <stdio.h>
#include <unistd.h>
#include "gnuscopeglobals.h"

/* --- structs --- */

struct gauss_out {
  float center;     // center of the centroid
  float fwhm;       // fwhm of the centroid
  float area;       // area of the centroid
  float chisqr;     // chisqr of the centroid
};

/* --- globals --- */

GtkWidget *messagetext;
//int histid[254],histsize[254],histpointer;
//int *histloc[254];
//char field1[254][40],field2[254][40],field3[254][40];
GtkWidget *graph;
GtkWidget *maintext;
int markers[4],binsize,offset;
int currentrange[2];
int intype,spectra,plottype;
int spectradisplayed[16];
GtkWidget *graphsdisplayed[16];

/* --- function declarations --- */

void GaussFitReturn(int lchannel, int uchannel,struct gauss_out *output);
void DisplayMultiGaussian(struct gauss_out *data,int num);
void PeakGrabber(int num);

/* --- fuctions from analysis.c --- */

int GetSumCounts(int a, int b);
float GetSumCenter (int a, int b, int counts);
float GetSumDeviation (int a, int b, int counts, float center);

/* --- functions --- */

/* PeakGrabber
 *
 * Searches for peaks and requests gaussians.  When done displays the peaks it found
 */
void PeakGrabber(int num) 
{
  int i,j,k,l;
  struct gauss_out fits[30];
  float chandiffs[8092];
  int checkchannels[8092];
  int chan_big_up[2],chan_big_down[2];

  /* --- begin by initializing everything --- */
  for ( i = 0; i < 8092; i++) {
    chandiffs[i] = 0;
    checkchannels[i] = 0;
  }
  for ( i = 0; i < 30; i++) {
    fits[i].center = fits[i].fwhm = fits[i].area = fits[i].chisqr = 0;
  }
  
  /* --- now that everything is initialized we need to get the differences between
     --- each set of channels --- */
  for (i = 0; i < histsize[spectra] - 1; i++) {
    checkchannels[i] = 1;
    chandiffs[i] = *(histloc[spectra] + i) - *(histloc[spectra] + i + 1);
  }

  /* --- now we need to enter the finding loop --- */
  for ( i = 0; i < num; i++) {
    /* --- start by finding the two channels with the biggests up and down 
       --- in the allowed channels --- */
    chan_big_up[1] = -1;
    chan_big_down[1] = 1;
    chan_big_up[0] = 0;
    chan_big_down[0] = 1;
    for ( j = 0; j < histsize[spectra] - 1; j++) {
      if (checkchannels[j]) {
	
      }
    }

  } // done with finding loop

}

/* GaussFitReturn 
 *
 * returns the relevant information about a gaussian fit given a range of channels
 */
void GaussFitReturn(int lchannel, int uchannel, struct gauss_out *output)
{
  float deviation,chiolddeviation,deldeviation,flagdeviation,chiold,delx;  
  float delxold,deldeviationold;
  float center;
  int i,j,k;
  int fixwflg,flagw;
  float chisqr, area,centerenergy, deviationenergy,correctedcounts;
  int exitflag;
  float bestfit[4];
  char dummystr[80];

  /* --- this will automatically fail if there is one channel --- */
  g_return_if_fail(lchannel == uchannel);
  
  /* --- just to be safe let's make sure that lchannel is less than uchannel --- */
  if (lchannel > uchannel) {
    lchannel = j;
    lchannel = uchannel;
    uchannel = j;
  }
  
  /* --- now let's initialize stuff --- */
  bestfit[0] = 1000000;
  chiolddeviation = 10000000;
  deldeviation = -0.05;
  flagw = 0;
  center = GetSumCenter(lchannel, uchannel, -1);
  deviation = GetSumDeviation(lchannel,uchannel,-1,center);  
  exitflag = 0;
  
  /* --- Start the analysis --- */
  while ((exitflag != 1) && ((deldeviation > 0.0001) || ((- deldeviation) > 0.0001 ))) {
    chiold = 1000000;
    delx = 0.1;
    chisqr = GaussChiSqr(localmarkers[0],localmarkers[1],center,deviation,&area);
    if (chisqr == -1) exitflag = 1;
    if (chisqr <= chiolddeviation) {
      chiolddeviation = chisqr;
      deviation = deviation + deldeviation;
    } else {
      deldeviationold = deldeviation;
      deldeviation = - deldeviation / 5;
      chiolddeviation = chisqr;
      deviation = deviation + deldeviation;
    }    
    deviation = deviation - deldeviationold;
    while ((delx > 0.0009) || (- delx > 0.0009)) {
      chisqr = GaussChiSqr(localmarkers[0],localmarkers[1],center,deviation,&area);
      if (chisqr == -1) exitflag = 1;
      if (chisqr <= chiold) {
        chiold = chisqr;
        center = center + delx;
      } else {
        delxold = delx;
        delx = - delx/5;
        chiold = chisqr;
        center = center + delx;
      }
    } 
    center = center - delxold;
    if (chisqr < bestfit[0]) {
      bestfit[0] = chisqr;
      bestfit[1] = center;
      bestfit[2] = deviation;
      bestfit[3] = area;
    }
  }
  
  output.chisqr = bestfit[0];
  output.center = bestfit[1]; // center has histogram count starting at zero
  output.fwhm = bestfit[2];  // fwhm/1.665
  output.area = bestfit[3];  // area / fwhm / 1.665
}

/* DisplayMultiGuassian
 *
 * Displays all the gaussians that were found in the gaussian search
 */
void DisplayMultiGaussian(struct gauss_out *data,int num)
{
  int i,j,k,l;
  int a,b;
  float stepsize;
  GdkPoint points;
  int max = 0;
  int bar_height,next_bar_height;
  float width,height;
  float column_width;
  float tempsum,tempavg;
  int localmarkers[2];
  long double rangewidth,currentscaling;
  float xchan,nextxchan;

  /* --- clear the anything that is already overlayed --- */
  if (gtk_graph_has_segments(graph)){
    gtk_graph_clear_segments(GTK_GRAPH(graph));
  }
  if (lastfitnumpoints) {
    LastFitClearPoints();
  }
  
  /* --- enter the loop in which everything gets rendered --- */
  for (l = 0; l < num; l++) {

    /* --- the range in which we will display the histogram is
       --- center +_ fwhm * 1.5 --- */
    localmarkers[1] = data[l].center + (data[l].fwhm * 1.665 * 1.5);
    localmarkers[0] = data[l].center - (data[l].fwhm * 1.665 * 1.5);

    /* --- Get height and width --- */
    height = graph->allocation.height - 31;
  
    if (binsize > 0) {
    
      /* --- find the max value --- */
      /* --- The max value can be modivied by the user 
	 --- using yscaleforce (yscale) and ybinforce(ybin)
      */
      tempsum = 0;
      tempavg = 0;
      if (yscaleforce == 1) {
	max = yscale;
      } else {
	for (i = currentrange[0]; i <= currentrange[1] ; i = i + binsize) {
	  tempsum = 0;
	  tempavg = 0;
	  for (j = 0; j < binsize; j++) {
	    tempsum = tempsum + *(histloc[spectra] + i + j);
        }
	  tempavg = tempsum/binsize;
	  if (max < tempsum) max = tempsum;
	}
      }
    }
    if (ybinforce == 1){
      if (ybin > 0) {
	max =(int)(max *  ybin) ;
      } else {
	if (ybin < 0) {
	  max = (int) (max /( - ybin));
	} else {
	  if (ybin == 0) {
            ybinforce = 0;
	  }
	}
      }
    }
    max = max * 1.1;
    if (max < 10) max = 10;
  
    currentmax = max;
    rangewidth = currentrange[1] - currentrange[0];
    if (rangewidth !=0){
      currentscaling = (long double) ((graph->allocation.width-100) / rangewidth);
    }
    
    /* --- Display each value --- */
    /* --- do this by figuring out which pixels on the screen are between the localmarkers[0-1]
       --- and then plot the gaussian between them. --- */
    if (binsize >  1) {
      a = (localmarkers[0] - currentrange[0]) * displayused + 100;
      b = (localmarkers[1] - currentrange[0]) * displayused + 100;
    } else {
      a = (localmarkers[0] - currentrange[0] + 0.5) * currentscaling + 100;
      b = (localmarkers[1] - currentrange[0] + 0.5) * currentscaling + 100;
    }
    
    for (i = a; i <= b ; i++) {
      /* --- now we have to convert back to get the right channel numbers --- */
      if (binsize > 1) {
	xchan = (i-100) /displayused + (float) currentrange[0];
	nextxchan = (i-99) /displayused + (float) currentrange[0];
      } else {
	if ((binsize == 1) && (currentscaling !=0)) {
	  xchan = (i-(100))/currentscaling  + currentrange[0] - .5;
	  nextxchan = (i-1 - (100))/currentscaling + currentrange[0] - .5;
	}
      }
      
      bar_height = ((area * exp(-(xchan - center)*(xchan-center)/deviation/deviation ))+background);
      LastFitAddPoint((float)xchan,(float)bar_height);
      if (binsize > 1) {
	points.x = i;
      } else {
	points.x = (i);
      }
      if (plottype == 2) {
	bar_height = (int) 1000 * log((double) bar_height);
      }
      if (plottype == 3) {
	bar_height = (int) sqrt((double) bar_height);
      }
      points.y = height - bar_height * height/max;
      gtk_graph_add_segment(GTK_GRAPH(graph),(GdkPoint *) &points);     
    }
  }
  Redraw();
}

/* --- misc crap --- */
