/* displaywindow.c
 * by:  John Pavan
 *
 * so you can click on the title of a graph and it will display
 */

/* --- includes --- */

#include <gtk/gtk.h>
#include "gtkgraph.h"
#include "gtktwodplot.h"
#include <stdio.h>
#include "gnuscopeglobals.h"
//--ddc trouble with function prototypes...
//#include "pgam.h"
#include "gnuscopefuncs.h"

/* --- structs --- */

/* --- globals --- */

//int histid[254],histsize[254],histpointer;
//int *histloc[254];
//char field1[254][40],field2[254][40],field3[254][40];
int numspectra;
int spectra;
int currentrange[2];
int xcurrentrange[2],ycurrentrange[2];
short int display_toggle[254];
short int twod_display_toggle[254];
GtkWidget *twoddisplayed[16];
short int twodspectradisplayed[16];
short int numtwoddisplayed;
int pgammax;
struct bigdata **big_data_info;
char twodtitles[254][80];

/* --- function declarations --- */

void PgamDisplaySelectionEntry(GtkWidget *widget, GtkWidget *entry);
void CloseWidget(GtkWidget *widget, gpointer *data);
void MultipleDisplayEntry(GtkWidget *widget, GtkWidget *entry);
void CreateDisplaySelectionWindow(int mode);
void ListBoxAddItemDisplay(GtkWidget *listbox, char *sText,int index);
void ListItemSelectedCallbackDisplay(GtkWidget *widget, gpointer *data);
void ListItemUnselectCallbackDisplay(GtkWidget *widget, gpointer *data);
void TwodListBoxAddItemDisplay(GtkWidget *listbox, char *sText,int index);
void TwodListItemSelectedCallbackDisplay(GtkWidget *widget, gpointer *data);
void TwodListItemUnselectCallbackDisplay(GtkWidget *widget, gpointer *data);
void DisplayWindowCallback(GtkWidget *widget, gpointer *data);
void TwodDisplayWindowCallback(GtkWidget *widget, gpointer *data);
void MultipleDisplaySameRangeEntry(GtkWidget *widget, GtkWidget *entry);
void MultiInOneEntry(GtkWidget *widget, GtkWidget *entry);

/* --- functions --- */

/* ListBoxAddItemDisplay
 *
 * Adds items to the display list box
 */
void ListBoxAddItemDisplay(GtkWidget *listbox, char *sText,int index)
{
  GtkWidget *item;

  item = gtk_list_item_new_with_label(sText);
  gtk_signal_connect(GTK_OBJECT(item),"select",
		     GTK_SIGNAL_FUNC(ListItemSelectedCallbackDisplay),
		     (int *) index);
  gtk_signal_connect(GTK_OBJECT(item),"deselect",
		     GTK_SIGNAL_FUNC(ListItemUnselectCallbackDisplay),
		     (int *) index);
  gtk_container_add(GTK_CONTAINER(listbox),item);
}

/* TwodListBoxAddItemDisplay
 * Adds a list item for the twoddisplay selection boxes
 */
void TwodListBoxAddItemDisplay(GtkWidget *listbox, char *sText, int index)
{
  GtkWidget *item;
  
  item = gtk_list_item_new_with_label(sText);
  gtk_signal_connect(GTK_OBJECT(item),"select",
		     GTK_SIGNAL_FUNC(TwodListItemSelectedCallbackDisplay),
		     (int *) index);
  gtk_signal_connect(GTK_OBJECT(item),"deselect",
		     GTK_SIGNAL_FUNC(TwodListItemUnselectCallbackDisplay),
		     (int *) index);
  gtk_container_add(GTK_CONTAINER(listbox),item);
}

/* TwodCreateDisplaySelectionWindow
 *
 * Creates a window with a list box for the purpose of selecting what to display
 * Currently does not support modes
 */
void TwodCreateDisplaySelectionWindow()
{
  GtkWidget *localwindow;
  GtkWidget *localbutton;
  GtkWidget *localvbox;
  GtkWidget *localhbox;
  GtkWidget *locallistbox;
  GtkWidget *localframe;
  GtkWidget *framehbox;
  GtkWidget *localentry;
  char dummystr[80];
  int i,j;

  /* --- idiot check --- */
  g_return_if_fail(pgammax != 0);

  /* --- Create the window and the appropriate stuff in the window --- */
  localwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(localwindow),"2D Histogram Display Selection");
  gtk_window_set_modal(GTK_WINDOW(localwindow),TRUE);

  /* --- create the localvbox --- */
  localvbox = gtk_vbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER(localwindow),localvbox);
  
  /* --- create the frame --- */
  localframe = gtk_frame_new("Select 2D histograms to display");
  gtk_container_add(GTK_CONTAINER(localvbox),localframe);

  /* --- make a horizontal box inside the frame --- */

  framehbox = gtk_hbox_new(TRUE,0);
  gtk_container_add(GTK_CONTAINER(localframe),framehbox);

  i = 0; // keep track of how many histograms have been listed

  while (i < pgammax) {
    if ((i % 10) == 0) {
      locallistbox = gtk_list_new();
      gtk_list_set_selection_mode(GTK_LIST(locallistbox),GTK_SELECTION_MULTIPLE);
      gtk_container_add(GTK_CONTAINER(framehbox),locallistbox);
    }
    sprintf(dummystr,"%d: %s",i+1,twodtitles[i]);
    TwodListBoxAddItemDisplay(locallistbox,dummystr,i);
    i++;
  }
  
  /* --- create an entry for entering information by hand --- */
  localentry = gtk_entry_new_with_max_length(80);
  gtk_container_add(GTK_CONTAINER(localvbox),localentry);
  gtk_signal_connect(GTK_OBJECT(localentry),"activate",
		     GTK_SIGNAL_FUNC(PgamDisplaySelectionEntry),localentry);
  gtk_signal_connect_object(GTK_OBJECT(localentry),"activate",
			    GTK_SIGNAL_FUNC(CloseWidget),(GtkObject *) localwindow);

  localbutton = gtk_button_new_with_label("Display");
  gtk_container_add(GTK_CONTAINER(localvbox),localbutton);
  gtk_signal_connect(GTK_OBJECT(localbutton),"clicked",
		     GTK_SIGNAL_FUNC(TwodDisplayWindowCallback),NULL);
  gtk_signal_connect_object(GTK_OBJECT(localbutton),"clicked",
			    GTK_SIGNAL_FUNC(CloseWidget),GTK_OBJECT(localwindow));

  localbutton = gtk_button_new_with_label("Cancel");
  gtk_container_add(GTK_CONTAINER(localvbox),localbutton);
  gtk_signal_connect_object(GTK_OBJECT(localbutton),"clicked",
			    GTK_SIGNAL_FUNC(CloseWidget),GTK_OBJECT(localwindow));
  
  gtk_widget_show_all(localwindow);
  gtk_widget_grab_focus(localentry);
}

/* CreateDisplaySelectionWindow
 *
 * Creates a window with a list box for the purpose of selecting what to display
 * mode: 0 = display full range, 1 = display current range, 2 = overlay in active graph
 */
void CreateDisplaySelectionWindow(int mode)
{
  GtkWidget *localwindow;
  GtkWidget *localbutton;
  GtkWidget *localvbox;
  GtkWidget *localhbox;
  GtkWidget *locallistbox;
  GtkWidget *localframe;
  GtkWidget *framehbox;
  GtkWidget *localentry;
  char dummystr[80];
  int i,j;

  /* --- idiot check --- */
  g_return_if_fail(histsize[0] > 0);

  /* --- make the window --- */

  localwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  switch (mode) {
  case 0:
    gtk_window_set_title(GTK_WINDOW(localwindow),"Display Full Domain");
    break;
  case 1:
    gtk_window_set_title(GTK_WINDOW(localwindow),"Display Current Domain");
    break;
  case 2:
    gtk_window_set_title(GTK_WINDOW(localwindow),"Overlay Histograms");
    break;
  }
  gtk_window_set_modal(GTK_WINDOW(localwindow),TRUE);

  /* --- create the localvbox --- */
  
  localvbox = gtk_vbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER(localwindow),localvbox);

  /* --- create the frame --- */

  localframe = gtk_frame_new("Select histograms to Display");
  gtk_container_add(GTK_CONTAINER(localvbox),localframe);

  /* --- create the horizontal box for the frame --- */

  framehbox = gtk_hbox_new(TRUE,0);
  gtk_container_add(GTK_CONTAINER(localframe),framehbox);

  i = 0; // to keep track of how many histograms have been listed;

  while (histsize[i] > 0) {
    if ((i % 10) == 0) {
      locallistbox = gtk_list_new();
      gtk_list_set_selection_mode(GTK_LIST(locallistbox),GTK_SELECTION_MULTIPLE);
      gtk_container_add(GTK_CONTAINER(framehbox),locallistbox);
    }
    sprintf(dummystr,"%d: %s",i+1,field2[i]);
    ListBoxAddItemDisplay(locallistbox,dummystr,i);
    i++;
  }
  
  /* --- create an entry for entering information by hand --- */

  localentry = gtk_entry_new_with_max_length(80);
  gtk_container_add(GTK_CONTAINER(localvbox),localentry);
  switch (mode) {
  case 1:
    gtk_signal_connect(GTK_OBJECT(localentry),"activate",
		       GTK_SIGNAL_FUNC(MultipleDisplaySameRangeEntry),localentry);
    break;
  case 2:
    gtk_signal_connect(GTK_OBJECT(localentry),"activate",
		       GTK_SIGNAL_FUNC(MultiInOneEntry),localentry);
    break;
  default:
    gtk_signal_connect(GTK_OBJECT(localentry),"activate",
		       GTK_SIGNAL_FUNC(MultipleDisplayEntry),localentry);
  }
  gtk_signal_connect_object(GTK_OBJECT(localentry),"activate",
			    GTK_SIGNAL_FUNC(CloseWidget),(GtkObject *) localwindow);

  /* --- create a localhbox for the display and cancel buttons --- */

  localhbox = gtk_hbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER(localvbox),localhbox);

  /* --- create the display button and cancel button --- */

  localbutton = gtk_button_new_with_label("Display");
  gtk_container_add(GTK_CONTAINER(localhbox),localbutton);
  gtk_signal_connect(GTK_OBJECT(localbutton),"clicked",
		     GTK_SIGNAL_FUNC(DisplayWindowCallback),(int *)mode);
  gtk_signal_connect_object(GTK_OBJECT(localbutton),"clicked",
		     GTK_SIGNAL_FUNC(CloseWidget),GTK_OBJECT(localwindow));

  localbutton = gtk_button_new_with_label("Cancel");
  gtk_container_add(GTK_CONTAINER(localhbox),localbutton);
  gtk_signal_connect_object(GTK_OBJECT(localbutton),"clicked",
		     GTK_SIGNAL_FUNC(CloseWidget),GTK_OBJECT(localwindow));

  gtk_widget_show_all(localwindow);
  gtk_widget_grab_focus(localentry);
}

/* TwodListItemSelectedCallbackDisplay
 * Interprets the result of the callback when a list item is selected
 */
void TwodListItemSelectedCallbackDisplay(GtkWidget *widget, gpointer *data)
{
  twod_display_toggle[(int) data] = 1;
}

/* ListItemSelectedCallbackDisplay
 *
 * Interprets the result of the callback when a list item is selected
 * mainly sets the toggle for that item to 1
 */
void ListItemSelectedCallbackDisplay(GtkWidget *widget, gpointer *data)
{
  display_toggle[(int) data] = 1;
}

/* TwodListItemUnselectedCallbackDisplay
 * Interprets the result of the callback when a list item is deselected
 */
void TwodListItemUnselectCallbackDisplay(GtkWidget *widget,gpointer *data)
{
  twod_display_toggle[(int)data] = 0;
}

/* ListItemUnselectedCallbackDisplay
 *
 * sets tht toggle for the list item to 0
 */
void ListItemUnselectCallbackDisplay(GtkWidget *widget, gpointer *data)
{
  display_toggle[(int)data] = 0;
}

/* TwodDisplayWindowCallback
 *
 * When the display window is activated it looks at the twod toggle list
 * and then sets up the appropriate information before attempting a display
 */
void TwodDisplayWindowCallback(GtkWidget *widget, gpointer *data)
{
  int i,j;
  int templist[16];
  int max;
  int which;
  int xmax,ymax;

  i = 0; j = 0;
  while ((j < 16) && (i < pgammax)) {
    if (twod_display_toggle[i]) {
      templist[j] = i;
      j++;
    }
    i++;
  }
  if (j > 0) {
    /* --- have to determine the size of the x and y displays --- */
    xmax = 0; ymax = 0;
    for (i = 0; i < j; i++) {
      if (xmax < big_data_info[templist[i]]->axes[0]) xmax = big_data_info[templist[i]]->axes[0];
      if (ymax < big_data_info[templist[i]]->axes[1]) ymax = big_data_info[templist[i]]->axes[1];
    }
    xcurrentrange[0] = 0;
    ycurrentrange[0] = 0;
    xcurrentrange[1] = xmax;
    ycurrentrange[1] = ymax;
    numtwoddisplayed = j;
    for (i = 0; i < j; i++) {
      twodspectradisplayed[i] = templist[i];
    }
    TwodDisplayCurrentRange();
    for (i = 0; i < j; i++) twod_display_toggle[i] = 0;
  }
}

/* DisplayWindowCallback
 *
 * When the display window is activated it looks at the toggle list and 
 * then sets up the appropriate information before attempting to display
 */
void DisplayWindowCallback(GtkWidget *widget, gpointer *data)
{
  int i,j,k;
  int templist[16];
  int mode;
  int max;
  int which;

  mode = (int) data;

  i = 0;
  j = 0;
  while ((j < 16) && (histsize[i] > 0)) {
    if (display_toggle[i]) {
      templist[j] = i;
      j++;
    }
    i++;
  }
  if (j > 0) {
    /* --- things are handled a little differently for each of the modes --- */
    switch (mode) {
    case 0:
      /* --- this is the display full range one --- */
      /* --- need to find the maxium channels --- */
      max = 0;
      for (i = 0; i < j; i++)
	if (max < histsize[i])
	  max = histsize[i];
      if (max < 0) max = 10;
      currentrange[0] = 0;
      currentrange[1] = max - 1;
      /* --- and then we just continue to the display current range option --- */
    case 1:
      numspectra = j;
      for (i = 0; i < j; i++) {
	spectradisplayed[i][0] = templist[i];
	for (k = 1; k < 10; k++)
	  spectradisplayed[i][k] = -1;
      }
      break;
    case 2:
      /* --- this is the option where we overlay --- */
      /* --- first get the maxium number of channels --- */
      max = 0;
      for (i = 0; i < j; i++)
	if (max < histsize[i])
	  max = histsize[i];
      if (max < 0) max = 10;
      currentrange[0] = 0;
      currentrange[1] = max - 1;
      /* --- now set up the graph --- */
      //numspectra = j;
      which = 0;
      if (numspectra > 1) {
	for (i = 0; i < numspectra; i++) {
	  if (spectra = spectradisplayed[i][0]) which = i;
	}
      }
      for (i = 0; i < 10; i++) 
	spectradisplayed[which][i] = -1;
      for (i = 0; i < j; i++) {
	spectradisplayed[which][i] = templist[i];
      }
      break;
    }

    DisplayCurrentRange();
    for (i = 0; i < j; i++) display_toggle[i] = 0;
  }
}
