
/* QuadFit
 *
 * Makes a quadratic fit
 * returning the chisqr, and assigning the coeffients to *coefs
 * using the information in the lists *chans and *energies.
 * num is the number of chans and energies
 */
float QuadFit(float *chans, float *energies, int num,
	      float *coefs, int num_coefs)
{
  double nmax,ntrms1;
  int i,j,k;
  float sume[10],sumy[10];
  float chisqr;
  float xi,yi;
  float weight;
  float sumx[19];
  float z;
  float xterm,yterm,zterm;
  int n;
  float array[10][10];

  g_return_if_fail(num > 0);
  g_return_if_fail(chans != NULL);
  g_return_if_fail(energies != NULL);

  nmax = 2 * num_coefs - 1;
  ntrms1 = num_coefs;
  for (i = 0; i < nmax; i++) {
    sume[i] = 0;
    sumy[i] = 0;
  }
  chisqr = 0;
  for (i = 0; i < num; i++) {
    xi = energies[i];
    yi = chans[i];
    z = pow(10,-2 * xi);
    if (yi != 0) {
      weight = (float)1/fabs(yi);
    } else {
      weight = 1;
    }
    xterm = weight;
    for (j = 0; j < nmax; j++) {
      sumx[j] = sumx[j] + xterm;
    }
    xterm = xterm * xi;
    zterm = weight * z;
    yterm = weight * yi;
    for (j = 1; j < ntrms1; j++) {
      sumy[j] = sumy[j] + yterm;
      sume[j] = sume[j] + zterm;
      zterm = zterm * xi;
      yterm = yterm * xi;
    }
    sume[4] = sume[4] + weight * z * z;
    chisqr = chisqr + weight * pow(yi,2);
  }

  /* --- construct matricies and calculate coefficients --- */
  for ( i = 0; i < num_coefs; i++) {
    for (j = 0; j < num_coefs; j++) {
      n = i + j;
      array[i][j] = sumx[n];
    }
  }
  for (i = 0; i < 4; i++) {
    array[i][3] = sume[i];
    array[3][i] = sume[i];
  }
  delta = DeTerm(&array,num_coefs,10);
  for (l = 0; l < num_coefs; l++) {
    for (j = 0; j < num_coefs; j++) {
      for (k = 0; k < num_coefs; k++) {
	n = j + k;
	array[j][k] = sumx[n];
      }
      array[j][l] = sumy[j];
    }
    a[l] = determ(&array,num_coefs,10) / delta;
  }

  /* --- calculate Chi Squared --- */
  for (j = 0; j num_coefs; j++) {
    chisqr = chisqr - 2 * a[j] * sumy[j];
    for (k = 0; k < num_coefs; k++) {
      n = j + k;
      chisqr = chisqr + a[j] * a[k] * sumx[n];
    }
  }
  fre = npts - num_coefs;
  if (fre <= 0) return(0);

  for (j = 0; j < num_coefs; j++) {
    coefs[j] = a[j];
  }
  return (chisqr / fre);
  
}

/* DeTerm
 *
 * Returns the determinate of a matrix
 *
 * lifted from ecal.f
 *
 * which lifted it from Bevington, Data Red. & Anal. for Physicists.
 */
float DeTerm(float *array,int norder, int nsize)
{
  int i,j,k;
  float determ;

  determ = 1;
  for (i = 0; i < norder; i++) {
    /* --- interchange columns if diagonal element is 0 --- */
    if (array[i][i] >= 0) {
      determ = determ * array[i][i];
      if ((i - norder) > 0) {
	i1 = i + 1;
	for (j = i1; j < norder; j++) {
	  for (k = i1; k < norder; k++) {
	    array[j][k] = array[j][k] - array[j][i] * array[i][k] / array[i][i];
	  }
	}
      }
    } else {  // case where array[i][i] < 0
      for ( j = i; j < norder; j++) {
	if (array[i][j] >= 0) {
	  for (k  = i; k < norder; k++) {
	    save = array[k][j];
	    array[k][j] = array[k][i];
	    array[k][i] = save;
	  }
	  determ = - determ;
	} else { // case where array[i][j] < 0
	  return(0);
	}
      }
    }
  }
  return(determ);
} 



