#include "filereading.h"

/* everything assumes default size for int is 4bytes */

void init_ornl(struct ornldir *, struct ornlheader *);
int read_fsuhead(struct fsuheader *, FILE *);
int read_fsudir(struct fsudir *, FILE *);
int write_fsuhead(struct fsuheader *, FILE *);
int write_fsudir(struct fsudir *, FILE *);

void init_ornl(struct ornldir *spkdir, struct ornlheader *spkhead) {
  int i;
  /* 
     reset the "ornl file pointer", which points to the position
     of various data in the file in units of "half-words" (2bytes!)
     to zero... 
  */
  
  ornlfile=0;
  
  /* initialize all the elements of the directory */
  
  strncpy(spkdir->datatype,"HHIRFSPK",8);
  spkdir->numspec=0;
  spkdir->nextrec=1025; /*assumes no more than 254 histograms!*/
  for(i=0;i<254;i++){
    spkdir->entry[i].id=0;
    spkdir->entry[i].location=0;
  } 
  
  /* initialize all the elements of the header having "static" values */
  spkhead->bytes=-4;
  spkhead->length=64;
  spkhead->dimensions=1;
  spkhead->record=0;
  spkhead->datalength=0;
  for(i=0;i<3;i++) spkhead->calibration[i]=0;
  strncpy(spkhead->label,"label test",12);
  strncpy(spkhead->reserved,"        ",8);
  strncpy(spkhead->timestamp,"            ",12);
  strncpy(spkhead->timestamp,"INVALID",12);
  return;
}

/*********************************************************************/

/* 
   some routines for reading/writing out the FSU directory and header
   records, which required special treatment due to byte boundary problems
   in defining the structures.
*/

int read_fsuhead(struct fsuheader *head, FILE *input){
  
  fread(&head->buffsize,4,1,input);    /*record starts with record size */
  fread(&head->numchn,2,1,input);      /*number channels in histogram*/
  fread(&head->itype,2,1,input);       /*type of histogram... must be 2! */
  fread(head->label,6,1,input);        /*histogram label*/
  fread(&head->truet,4,1,input);       /*truetime for ADC, or livetime*/
  fread(&head->buffcheck,4,1,input);   /*record ends with record size :) */
  
  if(head->buffsize!=head->buffcheck){
    printf("\n Error reading NSM file, exiting.\n");
    exit(-1);
  }
  
  return(0);
  
}

int read_fsudir(struct fsudir *dir, FILE *input){
  
  fread(&dir->buffsize,4,1,input);   /*each record starts with record size */
  fread(&dir->numrun,2,1,input);     /*the run number*/
  fread(&dir->numspec,2,1,input);    /*the number of spectra in the file*/
  fread(&dir->trutim,4,1,input);     /*the ACTUAL truetime */
  fread(dir->dat,9,1,input);         /*date field*/
  fread(dir->tim,8,1,input);         /*time field*/
  fread(&dir->buffcheck,4,1,input);  /*fortran file record ends with size :) */
  
  if(dir->buffsize!=dir->buffcheck){
    printf("\n Error reading NSM file, exiting.\n");
    exit(-1);
  }
  
  return(0);
  
}

int write_fsuhead(struct fsuheader *head, FILE *output){
  
  fwrite(&head->buffsize,4,1,output);    /*record starts with record size */
  fwrite(&head->numchn,2,1,output);      /*number channels in histogram*/
  fwrite(&head->itype,2,1,output);       /*type of histogram... must be 2! */
  fwrite(head->label,6,1,output);        /*histogram label*/
  fwrite(&head->truet,4,1,output);       /*truetime for ADC, or livetime*/
  fwrite(&head->buffcheck,4,1,output);   /*record ends with record size :) */
  
  return(0);
  
}

int write_fsudir(struct fsudir *dir, FILE *output){
  
  fwrite(&dir->buffsize,4,1,output);   /*each record starts with record size */
  fwrite(&dir->numrun,2,1,output);     /*the run number*/
  fwrite(&dir->numspec,2,1,output);    /*the number of spectra in the file*/
  fwrite(&dir->trutim,4,1,output);     /*the ACTUAL truetime */
  fwrite(dir->dat,9,1,output);         /*date field*/
  fwrite(dir->tim,8,1,output);         /*time field*/
  fwrite(&dir->buffcheck,4,1,output);  /*fortran record ends with size :) */
  return(0);  
}




