/* gretadevel.c
 *
 * by John Pavan
 *
 * Contains functions for development of GRETA stuff
 */

/* --- includes --- */
#include <gtk/gtk.h>
#include "gnuscopefuncs.h"
#include "gnuscopeglobals.h"
/* --- declarations --- */
#define GRETA_BUFFER_SIZE 2020

/* --- function declarations
   --- should be included in gnuscopefuncs.h --- */

/* --- functions --- */

/* GRETAReadTrace
 * 
 * Attempts to read in traces from the 12 bit card
 */
void GRETAReadTrace(char *sFilename)
{
  FILE *INPUT;
  char dummystr[120];
  guchar buffer[GRETA_BUFFER_SIZE];
  int i,j,k;
  
  /* --- start by trying to open the file --- */
  if ((INPUT = fopen(sFilename,"r")) != NULL) {
    /* --- if the file was successfully opened --- */
    /* --- read in a buffer --- */
    /* --- cycle through file until we run out of data --- */
    /* --- or fill up our histogram space --- */
    j = 0;
    while (j < 1028) {
      if (fread(buffer,1,GRETA_BUFFER_SIZE,INPUT) == 0) goto gretareadtraceoutofdata;
      histsize[j] = (GRETA_BUFFER_SIZE - 30) / 2;
      histloc[j] = (int *) malloc(sizeof(int) * histsize[0]);
      for (i = 32; i < GRETA_BUFFER_SIZE-2; i+=4) {
	histloc[j][(i-30)/2] = (uint) buffer[i+1] + 256 *(uint) buffer[i];
	histloc[j][(i-32)/2] = (uint) buffer[i+3] + 256 *(uint) buffer[i+2];
      }
      j++;
    }
      /* --- it must be closed --- */
  gretareadtraceoutofdata:
    fclose(INPUT);
  } else {
    /* --- if we were not able to open the file let the user know --- */
    printf("Unable top open %s\n",sFilename);
  }
}

/* GretaTraceToMatLab
 *
 * Converts trace data (standard GRETA format) to
 * a single column for use with matlab
 */
void GretaTraceToMatLab(char *sFilename,char *outFilename)
{
  FILE *INPUT,*OUTPUT;
  guchar inbuffer[GRETA_BUFFER_SIZE];
  char outbuffer[40000];
  char dummystr[120];
  int i,j,k;
  k = 0;
  
  /* --- start by trying to open the file --- */
  if ((INPUT = fopen(sFilename,"r")) != NULL) {
    /* --- now we need to open the output file for writting --- */
    if ((OUTPUT = fopen(outFilename,"w")) != NULL) {
      /* --- ok, now we want to keep reading in the file until it runs out --- */
      sprintf(outbuffer,"");
      while (fread(inbuffer,1,GRETA_BUFFER_SIZE,INPUT) == GRETA_BUFFER_SIZE) {
	for (i = 32; i < GRETA_BUFFER_SIZE-2; i+=4) {
	  sprintf(dummystr,"%e %d\n",k * 1e-8,
		  (uint) inbuffer[i+3] + 256 *(uint) inbuffer[i+2]);
	  strcat(outbuffer,dummystr);
	  k++;
	  sprintf(dummystr,"%e %d\n",k * 1e-8,
		  (uint) inbuffer[i+1] + 256 *(uint) inbuffer[i]);
	  strcat(outbuffer,dummystr);
	  k++;
	}
	fputs(outbuffer,OUTPUT);
	sprintf(outbuffer,"");
	for (i = 0; i < 1000; i++) {
	  sprintf(dummystr,"%e %d\n",k * 1e-8,
		  (uint) inbuffer[GRETA_BUFFER_SIZE-1] + 256 *(uint) inbuffer[GRETA_BUFFER_SIZE-2]);
	  strcat(outbuffer,dummystr);
	  k++;
	}
	fputs(outbuffer,OUTPUT);
	sprintf(outbuffer,"");
      }



    gretatracetomatlaboutofdata:
      /* --- close the output file --- */
      fclose(OUTPUT);
    } else {
      printf("Error:  Could not open file %s for writing\n",outFilename);
    }
    
    /* --- if it was opened it must be closed --- */
    fclose(INPUT);
  } else {
    /* --- if we could not open the file we must tell the user of the error ---*/
    printf("Error:  Could not open file %s\n",sFilename);
  }
}

/*GretaMSTToMatLab
 *
 * Converts the 36 channel 50 sample greta ascii files to sample + 36 columns for matlab
 */
void GretaMSTToMatLab(char *sFilename,char *outFilename)
{
  FILE *INPUT;
  FILE *OUTPUT;
  char inbuffer[100];
  char outbuffer[100000];
  char **bufferptr;
  int outbufferpos;
  int inbufsize;
  int i,j,k;
  float samples[36][50];

  k = 0;

  if ((INPUT = fopen(sFilename,"r")) != NULL) {
    if ((OUTPUT = fopen(outFilename,"w")) != NULL) {
      /* --- read in an inbuffer's worth --- */
      while (fgets(inbuffer,100,INPUT) != 0) {
	/* --- now we want to process this until we run out of input buffer --- */
	/* --- read in 36 channels of something --- */
	/* --- skip the total energy --- */
	/* --- now we start reading in --- */
	for (i = 0; i < 36; i++) {
	  /* --- skip the energy in that segment --- */
	  fgets(inbuffer,100,INPUT);
	  /* --- now read in the trace --- */
	  for (j = 0; j < 50; j++) {
	    fgets(inbuffer,100,INPUT);
	    sscanf(inbuffer,"%f",&samples[i][j]);
	    if (samples[i][j] == 10000) samples[i][j] = 0;
	  }
	}
	/* --- now we have read in the traces and stored them in samples --- */
	/* --- must write them out --- */
	outbufferpos = 0;
	for (j = 0; j < 50; j++) {
	  outbufferpos += sprintf(&outbuffer[outbufferpos],"%d ",k);
	  for (i = 0; i < 36; i++) {
	    outbufferpos += sprintf(&outbuffer[outbufferpos],"%f ",samples[i][j]);
	  }
	  outbufferpos += sprintf(&outbuffer[outbufferpos],"\n");
	  k++;
	}
	/* --- make some space --- */
	
	outbufferpos += sprintf(&outbuffer[outbufferpos],"%d ",k);
	for (i = 0; i < 36; i++) {
	  outbufferpos += sprintf(&outbuffer[outbufferpos],"%f ",0);
	}
	outbufferpos += sprintf(&outbuffer[outbufferpos],"\n");
	k+=50;

	outbufferpos += sprintf(&outbuffer[outbufferpos],"%d ",k);
	for (i = 0; i < 36; i++) {
	  outbufferpos += sprintf(&outbuffer[outbufferpos],"%f ",0);
	}
	outbufferpos += sprintf(&outbuffer[outbufferpos],"\n");
	k++;
	
	/* --- now write out the outbuffer --- */
	fwrite(outbuffer,sizeof(char),outbufferpos,OUTPUT);
      }

      /* --- we're done --- */
      fclose(OUTPUT);
    } else {
      printf("Error: Cannot open %s for writting.\n",outFilename);
    }
    fclose(INPUT);
  } else {
    printf("Error: cannot open %s for reading.\n",sFilename);
  }
}
