/* pgammatricies.c
 *
 * by John Pavan
 * 
 * For use with gnuscope
 *
 * contains the algorythims and the functions and subroutines
 * for dealing with the two-dimensional matricies
 * such as a particle-gamma square or a gamma-gamma triangle
 *
 * these differ from the ones for use with gs92 because
 * they are composed of floats instead of short ints.
 * this means they take up much more memory
 */

/* --- includes --- */

#include "pgam.h"
#include <stdio.h>
#include <sys/types.h>
#include <math.h>
#include <gtk/gtk.h>
#include <sys/stat.h>
#include <fcntl.h>

/* --- structs --- */

// the struct for storing the data is in pgam.h

/* --- globals --- */

struct pgam_matrix_info pgammatrixdata;

/* --- function delcarations --- */

void ReadPgamTwod(char *sfilename);

/* --- functions --- */

/* ReadPgamTwod
 *
 * Read in the PgamTwoddata from the file sFilename
 */
void ReadPgamTwod(char *sfilename)
{
  char dummystr[80];
  FILE *infile;

  /* --- open and read in the file --- */
  if ((infile = fopen(sFilename,"r")) != NULL) {
    fread(&pgammatrixdata.headbuffer[0],4,1,infile);
    fread(&pgammatrixdata.size,4,1,infile);
    fread(&pgammatrixdata.what,4,1,infile);
    fread(&pgammatrixdata.type,4,1,infile);
    fread(&pgammatrixdata.headbuffer[0],4,1,infile);
    /* --- make sure that the two head buffer's agree --- */
    if (pgammatrixdata.headbuffer[0] != pgammatrixdata.headbuffer[1]) {
      GetMessageDialog("Error reading file header.\n");
      goto closefile;
    }
    /* --- if the header was read in correctly, let's get the data --- */
    fread(&pgammatrixdata.databuffer[0],4,1,infile);
    /* --- allocate memory for the data --- */
    if (pgammatrixdata.data == NULL) {
      pgammatrixdata.data = (float *) malloc(pgammatrixdata.databuffer[0]);
    } else {
      pgammatrixdata.data = (float *) realloc(pgammatrixdata.data,
					      pgammatrixdata.databuffer[0]);
    }
    /* --- read in the data --- */
    fread(pgammatrixdata.data,1,pgammatrixdata.databuffer[0],infile);
    fread(&pgammatrixdata.databuffer[1],4,1,infile);
    /* --- make sure that the data buffer's agree --- */
    if (pgammatrixdata.databuffer[0] != pgammatrixdata.databuffer[1]) 
      GetMessageDialog("Error reading data from file.");

  closefile:
    fclose(infile);
  } // done 
  
}
